local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'
local MINOR_VERSION = type('20110228170318') == 'number' and '20110228170318' or date('%Y%m%d%H%M%S')
local MODULE_NAME = 'xp'

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:GetAddon(MAJOR_VERSION)

local module = addon:createModule(MODULE_NAME, 'AceEvent-3.0')

module.defaults = addon:merge(module.defaults, {
    startXP = 0,
    targetXP = nil,
})

module.optionsTable = addon:merge(module.optionsTable, {
    name = "XP Bar",
    type = 'group',
    args = {
        startXP = {
            name = "Start XP",
            desc = "Minimum XP value to display.",
            type = 'range',
            min = 0,
            max = nil,
            step = 1,
            bigStep = 1,
            get = 'getStartXP',
            set = 'setStartXP',
        },
        startLevel = {
            name = "Start Level",
            desc = "Minimum level to display.",
            type = 'range',
            min = 1,
            max = nil,
            --step = 1,
            bigStep = 1,
            get = 'getStartLevel',
            set = 'setStartLevel',
        },
    },
})

function module:getStartXP(info)
    return self:getOptions().startXP
end

function module:getStartXPMax()
    return self:getCurrentXP()
end

function module:setStartXP(info,value)
    self:getOptions().startXP = value
    self:Invalidate()
end

function module:getStartLevel(info)
    local startXP = self:getStartXP()
    local level = self:calcLevelForXP(startXP)
    return level
end

function module:getStartLevelMax()
    local startXPMax = self:getStartXPMax()
    local levelMax = self:calcLevelForXP(startXPMax)
    return levelMax
end

function module:setStartLevel(info,value)
    local prevLevel = floor(value)
    local prevLevelXP = self:calcXpForLevel(prevLevel)
    local nextLevel = ceil(value)
    if prevLevel < nextLevel then
        local nextLevelXP = self:calcXpForLevel(nextLevel)
        local startXP =  prevLevelXP + (value-prevLevel)*(nextLevelXP-prevLevelXP)
        self:setStartXP(info,startXP)
    else
        self:setStartXP(info,prevLevelXP)
    end
end


module.oldInitialize = module.OnInitialize
function module:OnInitialize()
    self:oldInitialize()
end

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('UPDATE_EXHAUSTION', 'Invalidate')
    self:RegisterEvent('PLAYER_XP_UPDATE', 'Invalidate')
end

function module:Redraw()
    if not self.invalid then
        return
    end
    self.invalid = nil

    --addon:Debug("Redraw")

    local xpOptions = self:getOptions()
    local barCache = self.cache

    local maxPlayerLevel = GetMaxPlayerLevel()
    if maxPlayerLevel == 0 then
        self:Invalidate()
        return
    end
    local currentGlobalXP = self:getCurrentXP()
    local startGlobalXP = xpOptions.startXP
    local targetGlobalXP = xpOptions.targetXP or self:calcXpForLevel(maxPlayerLevel)
    local restedGlobalXP = GetXPExhaustion()
    local currentGlobalXPRatio = (currentGlobalXP-startGlobalXP)/(targetGlobalXP-startGlobalXP)
    local restedGlobalXPRatio =  (restedGlobalXP               )/(targetGlobalXP-startGlobalXP)
    -- create textures
    assert(barCache and barCache.frame)
    local frame = barCache.frame
    -- texture for curent xp
    barCache.currentXPTexture = barCache.currentXPTexture or frame:CreateTexture(nil,'ARTWORK')
    local currentXPTexture = barCache.currentXPTexture
    currentXPTexture:ClearAllPoints()
    currentXPTexture:SetPoint('TOPLEFT')
    currentXPTexture:SetPoint('BOTTOM')
    currentXPTexture:SetWidth(currentGlobalXPRatio * frame:GetWidth())
    currentXPTexture:SetTexture(0,0,1,1)

    -- texture for rested xp
    barCache.restedXPTexture = barCache.restedXPTexture or frame:CreateTexture(nil,'ARTWORK')
    local restedXPTexture = barCache.restedXPTexture
    restedXPTexture:ClearAllPoints()
    restedXPTexture:SetPoint('TOP')
    restedXPTexture:SetPoint('BOTTOM')
    restedXPTexture:SetPoint('LEFT', currentXPTexture, 'RIGHT')
    restedXPTexture:SetWidth(restedGlobalXPRatio * frame:GetWidth())
    restedXPTexture:SetTexture(1,0,1,1)

    -- textures for levels
    barCache.levelTextures = barCache.levelTextures or {}
    local levelTextures = barCache.levelTextures
    for level = 1,maxPlayerLevel do
        local xpForLeveL = self:calcXpForLevel(level)
        if startGlobalXP <= xpForLeveL and xpForLeveL <= targetGlobalXP then
            local levelGlobalXPRatio = (xpForLeveL-startGlobalXP)/(targetGlobalXP-startGlobalXP)
            levelTextures[level] = levelTextures[level] or frame:CreateTexture(frame:GetName()..'Level'..level,'OVERLAY')
            local levelTexture = levelTextures[level]
            levelTexture:ClearAllPoints()
            levelTexture:SetPoint('TOP')
            levelTexture:SetPoint('BOTTOM')
            if levelGlobalXPRatio < 1.0 then
                levelTexture:SetPoint('LEFT',frame,'LEFT',levelGlobalXPRatio * frame:GetWidth(),0)
            else
                levelTexture:SetPoint('RIGHT')
            end
            levelTexture:SetWidth(1)
            levelTexture:SetTexture(1,1,1,1)
            levelTexture:Show()
        elseif levelTextures[level] then
            levelTextures[level]:Hide()
        end
    end

    --update options table
    self.optionsTable.args.startXP.max = self:getStartXPMax()
    self.optionsTable.args.startLevel.max = self:getStartLevelMax()
end

function module:getCurrentXP()
    local xpInCurrentLevel = UnitXP('player')
    local level = UnitLevel('player')
    return xpInCurrentLevel + self:calcXpForLevel(level)
end

function module:calcXpForLevel(level)
    local sum = 0
    for i=1,level-1 do
        sum = sum + XP_PER_LEVEL[i]
    end
    return sum
end

function module:calcLevelForXP(xp)
    local level = 1
    while xp > self:calcXpForLevel(level+1) do
        level = level + 1
    end
    local startXP = self:calcXpForLevel(level)
    assert(startXP <= xp,"level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    if startXP == xp then
        return level
    end
    local nextXP = self:calcXpForLevel(level+1)
    local fraction = (xp-startXP)/(nextXP-startXP)
    assert(0.0 <= fraction and fraction <= 1.0, "fraction=" .. fraction .. ", level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    return level + fraction
end

