local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:GetAddon(MAJOR_VERSION)

local sidePanel = LibStub:GetLibrary('LibSidePanel-1.0', false)

addon.barTemplate = {}
local barTemplate = addon.barTemplate

barTemplate.optionsTable = {
    name = nil,
    type = 'group',
    handler = nil,
    args = {
        enable = {
            name = "Enable",
            desc = "Enable this bar.",
            type = "toggle",
            get = 'getEnable',
            set = 'setEnable'
        },
        size = {
            name = "Size",
            desc = "Size (i.e, width or height) of the bar.",
            type = 'range',
            min = 1,
            max = 20,
            bigStep = 1,
            get = 'getSize',
            set = 'setSize',
        },
        priority = {
            name = "priority",
            desc = "Priority of the bar. Lower values move the frame outwards.",
            type = 'range',
            min = 1,
            max = 200,
            bigStep = 1,
            get = 'getPriority',
            set = 'setPriority',
        },
        side = {
            name = "Side",
            desc = "Side of the screen to place the bar.",
            type = 'select',
            values = { TOP = 'TOP', LEFT = 'LEFT', RIGHT = 'RIGHT', BOTTOM = 'BOTTOM' },
            get = 'getSide',
            set = 'setSide',
        },
    },
}

function barTemplate:getEnable(info)
    return self:IsEnabled()
end

function barTemplate:setEnable(info,value)
    --self:getOptions().enable = value
    if value then
        self:Enable()
    else
        self:Disable()
    end
end

function barTemplate:getSize(info)
    return self:getOptions().size
end

function barTemplate:setSize(info,value)
    self:getOptions().size = value
    self:updateBarPlacement()
end

function barTemplate:getPriority(info)
    return self:getOptions().priority
end

function barTemplate:setPriority(info,value)
    self:getOptions().priority = value
    self:updateBarPlacement()
end

function barTemplate:getSide(info)
    return self:getOptions().side
end

function barTemplate:setSide(info,value)
    self:getOptions().side = value
    self:updateBarPlacement()
end


barTemplate.defaults = {
    enable = true,
    side = 'BOTTOM',
    size = 10,
    priority = 100,
}

--- Called when the module is loaded
-- (Ace3 does this)
function barTemplate:OnInitialize()
    self.cache = {}
    local name = self:GetName()
    self.optionsTable.name = name
    self.optionsTable.handler = self
    addon.optionsTable.args[name] = self.optionsTable
    addon.defaults.profile[name] = self.defaults
end

--- Called when the addon is enabled.
function barTemplate:OnEnable()
    self:createBar()
end

--- Called when the addon is disabled
function barTemplate:OnDisable()
    self:destroyBar()
end

function barTemplate:getOptions()
    return addon.db.profile[self:GetName()]
end

--- Creates a new bar or enables an existing one.
-- @param barName used to fetch profile settings and used to index cache
function barTemplate:createBar()
    local barOptions = self:getOptions()
    assert(barOptions)
    self.cache = self.cache or {}
    local barCache = self.cache
    assert(barCache)

    barCache.frame = barCache.frame or CreateFrame('Frame', MAJOR_VERSION .. self:GetName() .. 'Bar', UIParent)
    local frame = barCache.frame
    -- create a texture
    frame.texture = frame.texture or frame:CreateTexture(nil, 'BACKGROUND')
    local texture = frame.texture
    texture:ClearAllPoints()
    texture:SetPoint('TOPLEFT')
    texture:SetPoint('BOTTOMRIGHT')
    texture:SetTexture(0,0,0,1)
    frame:Show()
    if not self:IsHooked(frame, 'OnSizeChanged') then
        self:HookScript(frame,'OnSizeChanged','Invalidate')
    end
    if not self:IsHooked(frame, 'OnUpdate') then
        self:HookScript(frame,'OnUpdate','Redraw')
    end
    self:updateBarPlacement()
end

function barTemplate:Invalidate(event)
    self.invalid = true
    --addon:Debug('Invalidate by ' .. tostring(event))
end

function barTemplate:Redraw()
    if not self.invalid then
        return
    end
    self.invalid = nil
end

function barTemplate:updateBarPlacement()
    local barOptions = self:getOptions()
    assert(barOptions)
    self.cache = self.cache or {}
    local barCache = self.cache
    assert(barCache)
    if self:IsEnabled() then
        addon:Debug("UpdatePlacement")
        local frame = barCache.frame
        if barOptions.side == 'TOP' or barOptions.side == 'BOTTOM' then
            frame:SetHeight(barOptions.size)
        else
            frame:SetWidth(barOptions.size)
        end
        sidePanel:Register(frame, barOptions.side, barOptions.priority)
        self:Invalidate()
    end
end

function barTemplate:destroyBar()
    local barOptions = self:getOptions()
    assert(barOptions)
    self.cache = self.cache or {}
    local barCache = self.cache
    assert(barCache)

    local frame = barCache.frame
    if not frame then
        return
    end

    sidePanel:Unregister(frame)
    frame:Hide()
    self:Unhook(frame,'OnSizeChanged')
    self:Unhook(frame,'OnUpdate')
end

function addon:createModule(name,...)
    module = addon:NewModule(name,barTemplate,'AceHook-3.0',...)
    return module
end
